Linux wireless drivers development
 help / color / mirror / Atom feed
From: Thomas Gleixner <tglx@kernel.org>
To: LKML <linux-kernel@vger.kernel.org>
Cc: David Woodhouse <dwmw2@infradead.org>,
	Miroslav Lichvar <mlichvar@redhat.com>,
	John Stultz <jstultz@google.com>, Stephen Boyd <sboyd@kernel.org>,
	Anna-Maria Behnsen <anna-maria@linutronix.de>,
	Frederic Weisbecker <frederic@kernel.org>,
	thomas.weissschuh@linutronix.de,
	Arthur Kiyanovski <akiyano@amazon.com>,
	Rodolfo Giometti <giometti@enneenne.com>,
	Vincent Donnefort <vdonnefort@google.com>,
	Marc Zyngier <maz@kernel.org>,
	Oliver Upton <oliver.upton@linux.dev>,
	kvmarm@lists.linux.dev, Oliver Upton <oupton@kernel.org>,
	Richard Cochran <richardcochran@gmail.com>,
	netdev@vger.kernel.org, Takashi Iwai <tiwai@suse.com>,
	Miri Korenblit <miriam.rachel.korenblit@intel.com>,
	Johannes Berg <johannes.berg@intel.com>,
	Jacob Keller <jacob.e.keller@intel.com>,
	Tony Nguyen <anthony.l.nguyen@intel.com>,
	Saeed Mahameed <saeedm@nvidia.com>,
	Peter Hilber <peter.hilber@oss.qualcomm.com>,
	"Michael S. Tsirkin" <mst@redhat.com>,
	virtualization@lists.linux.dev, linux-wireless@vger.kernel.org,
	linux-sound@vger.kernel.org
Subject: [patch 01/24] timekeeping: Provide ktime_get_snapshot_id()
Date: Tue, 26 May 2026 19:13:33 +0200	[thread overview]
Message-ID: <20260526171222.769770418@kernel.org> (raw)
In-Reply-To: 20260526165826.392227559@kernel.org

ktime_get_snapshot() provides a snapshot of the underlying clocksource
counter value and the corresponding CLOCK_MONOTONIC_RAW, CLOCK_REALTIME and
CLOCK_BOOTTIME timestamps.

There is no usage of CLOCK_REALTIME and CLOCK_BOOTTIME at the same time and
CLOCK_BOOTTIME support was just added for the ARM64 KVM tracing mechanism,
which needs CLOCK_BOOTTIME and the underlying clocksource counter value.

ktime_get_snapshot() is also not suitable for usage with CLOCK_AUX, but
that's a prerequisite to support PTP hardware timestamping for CLOCK_AUX
steering.

As a first step, rename ktime_get_snapshot() to ktime_get_snapshot_id(),
which now takes a clockid argument to select the clock which needs to be
captured. The result is stored in system_time_snapshot::sys, which will
replace the system_time_snapshot::real/boot members once all usage sites
have been converted.

ktime_get_snapshot() is a simple wrapper which hands in CLOCK_REALTIME as
clockid argument for the conversion period. That means CLOCK_REALTIME is
now captured twice, but that redunancy is only temporary.

No functional change vs. current users of ktime_get_snapshot()

Signed-off-by: Thomas Gleixner <tglx@kernel.org>
---
 include/linux/timekeeping.h |   29 ++++++++++-----
 kernel/time/timekeeping.c   |   84 +++++++++++++++++++++++++++++++++-----------
 2 files changed, 84 insertions(+), 29 deletions(-)

--- a/include/linux/timekeeping.h
+++ b/include/linux/timekeeping.h
@@ -276,24 +276,28 @@ static inline bool ktime_get_aux_ts64(cl
 #endif
 
 /**
  * struct system_time_snapshot - simultaneous raw/real time capture with
  *				 counter value
  * @cycles:	Clocksource counter value to produce the system times
  * @real:	Realtime system time
  * @boot:	Boot time
  * @raw:	Monotonic raw system time
  * @cs_id:	Clocksource ID
  * @clock_was_set_seq:	The sequence number of clock-was-set events
  * @cs_was_changed_seq:	The sequence number of clocksource change events
+ * @valid:		True if the snapshot is valid
  */
 struct system_time_snapshot {
 	u64			cycles;
+	ktime_t			sys;
 	ktime_t			real;
 	ktime_t			boot;
 	ktime_t			raw;
 	enum clocksource_ids	cs_id;
 	unsigned int		clock_was_set_seq;
 	u8			cs_was_changed_seq;
+	u8			valid;
 };
 
 /**
@@ -341,9 +345,16 @@ extern int get_device_system_crosststamp
 			struct system_device_crosststamp *xtstamp);
 
 /*
- * Simultaneously snapshot realtime and monotonic raw clocks
+ * Simultaneously snapshot a given clock with MONOTONIC_RAW and the underlying
+ * clocksource counter value.
  */
-extern void ktime_get_snapshot(struct system_time_snapshot *systime_snapshot);
+extern bool ktime_get_snapshot_id(struct system_time_snapshot *systime_snapshot,
+				  clockid_t clock_id);
+
+static inline void ktime_get_snapshot(struct system_time_snapshot *systime_snapshot)
+{
+	WARN_ON_ONCE(!ktime_get_snapshot_id(systime_snapshot, CLOCK_REALTIME));
+}
 
 /*
  * Persistent clock related interfaces
@@ -276,24 +276,28 @@ static inline bool ktime_get_aux_ts64(cl
 
 /**
- * struct system_time_snapshot - simultaneous raw/real time capture with
- *				 counter value
- * @cycles:	Clocksource counter value to produce the system times
- * @real:	Realtime system time
- * @boot:	Boot time
- * @raw:	Monotonic raw system time
- * @cs_id:	Clocksource ID
+ * struct system_time_snapshot - Simultaneous time capture of CLOCK_MONOTONIC_RAW,
+ *				 a selected CLOCK_* and the clocksource counter value
+ * @cycles:		Clocksource counter value to produce the system times
+ * @sys:		The system time of the selected CLOCK ID
+ * @real:		Realtime system time
+ * @boot:		Boot time
+ * @raw:		Monotonic raw system time
+ * @cs_id:		Clocksource ID
  * @clock_was_set_seq:	The sequence number of clock-was-set events
  * @cs_was_changed_seq:	The sequence number of clocksource change events
--- a/kernel/time/timekeeping.c
+++ b/kernel/time/timekeeping.c
@@ -1183,43 +1183,87 @@ noinstr time64_t __ktime_get_real_second
 }
 
 /**
  * ktime_get_snapshot - snapshots the realtime/monotonic raw clocks with counter
  * @systime_snapshot:	pointer to struct receiving the system time snapshot
  */
-void ktime_get_snapshot(struct system_time_snapshot *systime_snapshot)
+bool ktime_get_snapshot_id(struct system_time_snapshot *systime_snapshot, clockid_t clock_id)
 {
-	struct timekeeper *tk = &tk_core.timekeeper;
+	ktime_t base_raw, base_sys, offs_sys, *offs, offs_zero = 0;
+	u64 nsec_raw, nsec_sys, now;
+	struct timekeeper *tk;
+	struct tk_data *tkd;
 	unsigned int seq;
-	ktime_t base_raw;
 	ktime_t base_real;
 	ktime_t base_boot;
-	u64 nsec_raw;
-	u64 nsec_real;
-	u64 now;
 
-	WARN_ON_ONCE(timekeeping_suspended);
+	/* Invalidate the snapshot for all failure cases */
+	systime_snapshot->valid = false;
+
+	if (WARN_ON_ONCE(timekeeping_suspended))
+		return false;
+
+	switch (clock_id) {
+	case CLOCK_REALTIME:
+		tkd = &tk_core;
+		offs = &tk_core.timekeeper.offs_real;
+		break;
+	/* Map RAW to MONOTONIC so the loop below is trivial */
+	case CLOCK_MONOTONIC_RAW:
+	case CLOCK_MONOTONIC:
+		tkd = &tk_core;
+		offs = &offs_zero;
+		break;
+	case CLOCK_BOOTTIME:
+		tkd = &tk_core;
+		offs = &tk_core.timekeeper.offs_boot;
+		break;
+	default:
+		WARN_ON_ONCE(1);
+		return false;
+	}
+
+	tk = &tkd->timekeeper;
 
 	do {
-		seq = read_seqcount_begin(&tk_core.seq);
+		seq = read_seqcount_begin(&tkd->seq);
+
 		now = tk_clock_read(&tk->tkr_mono);
 		systime_snapshot->cs_id = tk->tkr_mono.clock->id;
 		systime_snapshot->cs_was_changed_seq = tk->cs_was_changed_seq;
 		systime_snapshot->clock_was_set_seq = tk->clock_was_set_seq;
-		base_real = ktime_add(tk->tkr_mono.base,
-				      tk_core.timekeeper.offs_real);
-		base_boot = ktime_add(tk->tkr_mono.base,
-				      tk_core.timekeeper.offs_boot);
+
+		base_sys = tk->tkr_mono.base;
+		offs_sys = *offs;
 		base_raw = tk->tkr_raw.base;
-		nsec_real = timekeeping_cycles_to_ns(&tk->tkr_mono, now);
-		nsec_raw  = timekeeping_cycles_to_ns(&tk->tkr_raw, now);
-	} while (read_seqcount_retry(&tk_core.seq, seq));
+
+		/* Kept around until the callers are fixed up */
+		base_real = ktime_add(base_sys, tk_core.timekeeper.offs_real);
+		base_boot = ktime_add(base_sys, tk_core.timekeeper.offs_boot);
+
+		nsec_sys = timekeeping_cycles_to_ns(&tk->tkr_mono, now);
+		nsec_raw = timekeeping_cycles_to_ns(&tk->tkr_raw, now);
+	} while (read_seqcount_retry(&tkd->seq, seq));
 
 	systime_snapshot->cycles = now;
-	systime_snapshot->real = ktime_add_ns(base_real, nsec_real);
-	systime_snapshot->boot = ktime_add_ns(base_boot, nsec_real);
+	systime_snapshot->sys = ktime_add_ns(base_sys, offs_sys + nsec_sys);
+	systime_snapshot->real = ktime_add_ns(base_real, nsec_sys);
+	systime_snapshot->boot = ktime_add_ns(base_boot, nsec_sys);
 	systime_snapshot->raw = ktime_add_ns(base_raw, nsec_raw);
+
+	/*
+	 * Special case for PTP. Just transfer the raw time into sys,
+	 * so the call sites can consistently use snap::sys.
+	 */
+	if (clock_id == CLOCK_MONOTONIC_RAW)
+		systime_snapshot->sys = systime_snapshot->raw;
+	/* Tell the consumer that this snapshot is valid */
+	systime_snapshot->valid = true;
+	return true;
 }
-EXPORT_SYMBOL_GPL(ktime_get_snapshot);
+EXPORT_SYMBOL_GPL(ktime_get_snapshot_id);
 
 /* Scale base by mult/div checking for overflow */
 static int scale64_check_overflow(u64 mult, u64 div, u64 *base)
@@ -1183,43 +1183,87 @@ noinstr time64_t __ktime_get_real_second
 
 /**
- * ktime_get_snapshot - snapshots the realtime/monotonic raw clocks with counter
- * @systime_snapshot:	pointer to struct receiving the system time snapshot
+ * ktime_get_snapshot_id -  Simultaneously snapshot a given clock ID with
+ *			    CLOCK_MONOTONIC_RAW and the underlying
+ *			    clocksource counter value.
+ * @systime_snapshot:	Pointer to struct receiving the system time snapshot
+ * @clock_id:		The clock ID to snapshot
  */


  reply	other threads:[~2026-05-26 17:13 UTC|newest]

Thread overview: 55+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2026-05-26 17:13 [patch 00/24] timekeeping/ptp: Expand snapshot functionality Thomas Gleixner
2026-05-26 17:13 ` Thomas Gleixner [this message]
2026-05-26 21:41   ` [patch 01/24] timekeeping: Provide ktime_get_snapshot_id() Jacob Keller
2026-05-26 23:39   ` David Woodhouse
2026-05-26 17:13 ` [patch 02/24] timekeeping: Use system_time_snapshot::sys instead of ::real Thomas Gleixner
2026-05-26 23:40   ` [PATCH 2/24] " David Woodhouse
2026-05-26 17:13 ` [patch 03/24] pps: generators: Use ktime_get_real_ts64() instead of ktime_get_snapshot() Thomas Gleixner
2026-05-26 23:42   ` [PATCH 3/24] " David Woodhouse
2026-05-26 17:13 ` [patch 04/24] pps: Convert to ktime_get_snapshot_id() Thomas Gleixner
2026-05-26 23:42   ` [PATCH 4/24] " David Woodhouse
2026-05-26 17:13 ` [patch 05/24] KVM: arm64: Use ktime_get_snapshot_id() to retrieve CLOCK_BOOTTIME Thomas Gleixner
2026-05-26 23:42   ` [PATCH 5/24] " David Woodhouse
2026-05-26 17:13 ` [patch 06/24] KVM: arm64: Use ktime_get_snapshot_id() to snapshot CLOCK_REALTIME Thomas Gleixner
2026-05-26 23:43   ` [PATCH 6/24] " David Woodhouse
2026-05-26 17:14 ` [patch 07/24] ptp: ptp_vmclock: Convert to ktime_get_snapshot_id() Thomas Gleixner
2026-05-26 23:43   ` [PATCH 7/24] " David Woodhouse
2026-05-26 17:14 ` [patch 08/24] timekeeping: Remove system_time_snapshot::real/boot Thomas Gleixner
2026-05-26 21:49   ` Jacob Keller
2026-05-26 21:52     ` Jacob Keller
2026-05-26 23:44   ` [PATCH 8/24] " David Woodhouse
2026-05-26 17:14 ` [patch 09/24] timekeeping: Add CLOCK_AUX support for ktime_get_snapshot_id() Thomas Gleixner
2026-05-26 23:44   ` [PATCH 9/24] " David Woodhouse
2026-05-26 17:14 ` [patch 10/24] timekeeping: Add system_counterval_t to struct system_device_crosststamp Thomas Gleixner
2026-05-26 23:45   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 11/24] timekeeping: Add CLOCK ID to system_device_crosststamp Thomas Gleixner
2026-05-26 23:46   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 12/24] wifi: iwlwifi: Adopt PTP cross timestamps to core changes Thomas Gleixner
2026-05-26 23:47   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 13/24] ice/ptp: Use provided clock ID for history snapshot Thomas Gleixner
2026-05-26 21:51   ` Jacob Keller
2026-05-26 23:49   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 14/24] igc: " Thomas Gleixner
2026-05-26 21:51   ` Jacob Keller
2026-05-26 23:50   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 15/24] net/mlx5: " Thomas Gleixner
2026-05-26 23:51   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 16/24] virtio_rtc: " Thomas Gleixner
2026-05-26 23:51   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 17/24] timekeeping: Remove ktime_get_snapshot() Thomas Gleixner
2026-05-26 23:52   ` [PATCH " David Woodhouse
2026-05-26 17:14 ` [patch 18/24] timekeeping: Prepare for cross timestamps on arbitrary clock IDs Thomas Gleixner
2026-05-26 23:54   ` [PATCH " David Woodhouse
2026-05-26 17:15 ` [patch 19/24] ptp: Use system_device_crosststamp::sys_systime Thomas Gleixner
2026-05-26 23:54   ` [PATCH " David Woodhouse
2026-05-26 17:15 ` [patch 20/24] wifi: iwlwifi: " Thomas Gleixner
2026-05-26 23:55   ` [PATCH " David Woodhouse
2026-05-26 17:15 ` [patch 21/24] ALSA: hda/common: " Thomas Gleixner
2026-05-26 23:55   ` [PATCH " David Woodhouse
2026-05-26 17:15 ` [patch 22/24] timekeeping: Remove system_device_crosststamp::sys_realtime Thomas Gleixner
2026-05-26 23:56   ` [PATCH " David Woodhouse
2026-05-26 17:15 ` [patch 23/24] timekeeping: Add support for AUX clock cross timestamping Thomas Gleixner
2026-05-26 23:56   ` [PATCH " David Woodhouse
2026-05-26 17:15 ` [patch 24/24] ptp: Switch to ktime_get_snapshot_id() for pre/post timestamps Thomas Gleixner
2026-05-26 23:57   ` [PATCH " David Woodhouse
2026-05-26 21:56 ` [patch 00/24] timekeeping/ptp: Expand snapshot functionality Jacob Keller

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20260526171222.769770418@kernel.org \
    --to=tglx@kernel.org \
    --cc=akiyano@amazon.com \
    --cc=anna-maria@linutronix.de \
    --cc=anthony.l.nguyen@intel.com \
    --cc=dwmw2@infradead.org \
    --cc=frederic@kernel.org \
    --cc=giometti@enneenne.com \
    --cc=jacob.e.keller@intel.com \
    --cc=johannes.berg@intel.com \
    --cc=jstultz@google.com \
    --cc=kvmarm@lists.linux.dev \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-sound@vger.kernel.org \
    --cc=linux-wireless@vger.kernel.org \
    --cc=maz@kernel.org \
    --cc=miriam.rachel.korenblit@intel.com \
    --cc=mlichvar@redhat.com \
    --cc=mst@redhat.com \
    --cc=netdev@vger.kernel.org \
    --cc=oliver.upton@linux.dev \
    --cc=oupton@kernel.org \
    --cc=peter.hilber@oss.qualcomm.com \
    --cc=richardcochran@gmail.com \
    --cc=saeedm@nvidia.com \
    --cc=sboyd@kernel.org \
    --cc=thomas.weissschuh@linutronix.de \
    --cc=tiwai@suse.com \
    --cc=vdonnefort@google.com \
    --cc=virtualization@lists.linux.dev \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox